"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var utils_1 = require("../utils");
var DuplicateDependencyError_1 = require("../Errors/DuplicateDependencyError");
var UnknownDependencyError_1 = require("../Errors/UnknownDependencyError");
var MultiDependencyError_1 = require("../Errors/MultiDependencyError");
var Singleton_1 = require("../Lifetime/Singleton");
var Scoped_1 = require("../Lifetime/Scoped");
var Transient_1 = require("../Lifetime/Transient");
var ServiceProvider_1 = require("../ServiceProvider");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection(ProviderTemplate) {
        Object.defineProperty(this, "template", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "dependencyToProvider", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.template = new ProviderTemplate();
        this.dependencyToProvider = Object.keys(this.template).reduce(function (a, b) {
            a[b.toLowerCase()] = b;
            return a;
        }, {});
    }
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, options) {
            if (!this.tryAdd(Lifetime, options)) {
                var name_1 = this.resolvePropertyConstructor(options)[0];
                throw new DuplicateDependencyError_1.DuplicateDependencyError(name_1);
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAdd", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, options) {
            var _a = this.resolvePropertyConstructor(options), name = _a[0], factory = _a[1], dependency = _a[2];
            if (!name)
                throw new UnknownDependencyError_1.UnknownDependencyError((dependency === null || dependency === void 0 ? void 0 : dependency.name) || '');
            if (this.lifetimes[name])
                return false;
            this.lifetimes[name] = new Lifetime(name, factory);
            return true;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (item) {
            return this.lifetimes[this.resolveProperty(item)];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replace", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, options) {
            var name = this.resolvePropertyConstructor(options)[0];
            this.remove(name);
            this.add(Lifetime, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (item) {
            var name = this.resolveProperty(item);
            if (!this.lifetimes[name])
                return false;
            delete this.lifetimes[name];
            return true;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var _this = this;
            var createContext = function (context) {
                var provider = new ServiceProvider_1.InternalServiceProvider(_this, createContext, { trail: {} }, context);
                Object.keys(_this.template).forEach(function (name) {
                    Object.defineProperty(provider, name, { get: function () { return provider.getService(name); } });
                });
                return provider;
            };
            return createContext();
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "validate", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var serviceProvider = this.build();
            var keys = Object.keys(this.template);
            var unresolved = [];
            for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                var key = keys_1[_i];
                try {
                    serviceProvider.getService(key);
                }
                catch (e) {
                    unresolved.push(e);
                }
            }
            if (unresolved.length === 0)
                return;
            throw new MultiDependencyError_1.MultiDependencyError(unresolved);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "resolveProperty", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (item, dependency) {
            if (item)
                return typeof item === 'string' ? item : item((0, utils_1.properties)(this.template));
            if (dependency)
                return this.dependencyToProvider[dependency.name.toLowerCase()];
            throw new UnknownDependencyError_1.UnknownDependencyError('<missing arguments>');
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "resolvePropertyConstructor", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            if (typeof options === 'function') {
                return [this.resolveProperty(undefined, options), function (p) { return new options(p); }, options];
            }
            else if (options.dependency) {
                var dependency_1 = options.dependency, selector = options.selector;
                return [this.resolveProperty(selector, dependency_1), function (p) { return new dependency_1(p); }, dependency_1];
            }
            else {
                var selector = options.selector, factory = options.factory;
                return [this.resolveProperty(selector), factory, undefined];
            }
        }
    });
    /**
     * @param options defaults to 'provider'
     */
    Object.defineProperty(ServiceCollection.prototype, "addProvider", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            if (options === void 0) { options = 'provider'; }
            this.addScoped({ factory: function (p) { return p.createScoped(); }, selector: options });
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.add(Singleton_1.Singleton, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.add(Scoped_1.Scoped, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.add(Transient_1.Transient, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.tryAdd(Singleton_1.Singleton, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.tryAdd(Scoped_1.Scoped, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.tryAdd(Transient_1.Transient, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.replace(Singleton_1.Singleton, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.replace(Scoped_1.Scoped, options);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            return this.replace(Transient_1.Transient, options);
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
