"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalServiceProvider = void 0;
var types_1 = require("./Errors/types");
var CircularDependencyError_1 = require("./Errors/CircularDependencyError");
var ExistenceDependencyError_1 = require("./Errors/ExistenceDependencyError");
var InternalServiceProvider = /** @class */ (function () {
    function InternalServiceProvider(services, create, validation, context) {
        Object.defineProperty(this, "_", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._ = {
            scope: context,
            validation: validation,
            services: services,
            create: create,
        };
    }
    Object.defineProperty(InternalServiceProvider.prototype, "getService", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (nameSelector) {
            var _a = this._, services = _a.services, scope = _a.scope, create = _a.create;
            var name = services.resolveProperty(nameSelector);
            try {
                var actual = scope ? this : create({});
                var lifetime = services.get(name);
                if (lifetime)
                    return lifetime.provide(actual);
            }
            catch (e) {
                var error = e;
                switch (error.type) {
                    case types_1.DependencyErrorType.Circular:
                        // When error is detected we can only say lifetime === cause
                        // Letting it throw 1 dependency up we can find the dependencies causing the issue
                        if (error.lifetime !== error.cause)
                            throw e;
                        throw new CircularDependencyError_1.CircularDependencyError(error.lifetime, name);
                    default:
                        throw e;
                }
            }
            throw new ExistenceDependencyError_1.ExistenceDependencyError(name);
        }
    });
    /**
     * Returns a new provider which stays within scope but resets validation
     */
    Object.defineProperty(InternalServiceProvider.prototype, "createScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var _a = this._, create = _a.create, scope = _a.scope;
            return create(scope);
        }
    });
    return InternalServiceProvider;
}());
exports.InternalServiceProvider = InternalServiceProvider;
